import Flutter
import UIKit
import Photos
import ZLPhotoBrowser

public class SwiftXImagePickerPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "x_image_picker", binaryMessenger: registrar.messenger())
    let instance = SwiftXImagePickerPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    if(call.method == "pick_image"){
        if let args = call.arguments as? Dictionary<String, Any>,
           let recordVideoSecond = args["recordVideoSecond"] as? Int,
           let maxSelectNum = args["maxSelectNum"] as? Int,
           let type = args["type"] as? String,
           let isCamera = args["isCamera"] as? Bool{
            
            let config = ZLPhotoConfiguration.default();
            config.allowSelectImage = type == "Image";
            config.allowSelectVideo = type == "Video";
            
            config.maxSelectCount = maxSelectNum;
            config.allowTakePhoto = type == "Image" && isCamera;
            config.allowRecordVideo = type == "Video" && isCamera;
            config.maxRecordDuration = recordVideoSecond;
            config.maxSelectVideoDuration = recordVideoSecond;
            config.languageType = .chineseSimplified
            config.allowSelectGif = false
            
            let ps = ZLPhotoPreviewSheet()
            var resArr = [Dictionary<String, Any>]()
            let group = DispatchGroup()
            
            ps.selectImageBlock = {(images, assets, isOriginal) in
                for(index, asset) in assets.enumerated(){
                    group.enter()
                    
                    if(asset.mediaType == PHAssetMediaType.image){
                        resArr.append(self.resolveImage(image: images[index], asset: asset))
                        group.leave()
                    }
                    
                    if(asset.mediaType == PHAssetMediaType.video){
                        self.resolveVideo(asset: asset) { dic in
                            resArr.append(dic)
                            group.leave()
                        }
                    }
                }
                
                group.notify(queue: .main){
                    result(resArr)
                }
            }
            
            let vc = UIApplication.shared.delegate!.window!!.rootViewController!;
            
            ps.showPhotoLibrary(sender: vc);
        }
        
    }
  }
    
    func resolveImage(image: UIImage, asset: PHAsset) -> Dictionary<String, Any>{
        var dic = [String: Any]()
        
        let data = image.jpegData(compressionQuality: 1)
        let path = createFile(data: data)
        
        dic.updateValue(path, forKey: "path")
        dic.updateValue(asset.pixelWidth, forKey: "width")
        dic.updateValue(asset.pixelHeight, forKey: "height")
        return dic;
    }
    
    func resolveVideo(asset: PHAsset, resultHandle: @escaping (Dictionary<String, Any>) -> Void) -> Void {
        let options = PHVideoRequestOptions.init()
        options.version = .original
        options.isNetworkAccessAllowed = true
        options.deliveryMode = .automatic
        
        let manager = PHImageManager.default()
        manager.requestAVAsset(forVideo: asset, options: options) { avAsset, audioMix, info in
            let videoUrl = avAsset as! AVURLAsset
            let url = videoUrl.url
            
            let urlStr = url.absoluteString
            let path = (urlStr as NSString).substring(from: 7);
            
            var dic = [String: Any]()
            dic.updateValue(path, forKey: "path")
            dic.updateValue(asset.pixelWidth, forKey: "width")
            dic.updateValue(asset.pixelHeight, forKey: "height")
            
            if let thumb = self.getVideoThumbPath(url: path){
                let thumbData = thumb.jpegData(compressionQuality: 1)
                let thumbPath = self.createFile(data: thumbData)
                
                dic.updateValue("coverPath", forKey: thumbPath)
            }
            
            resultHandle(dic)
        }
        
    }
    
    func createFile(data: Data?) -> String {
        let uuid = UUID().uuidString
        let tmpDir = NSTemporaryDirectory()
        let fileName = "\(tmpDir)image_picker_\(uuid)"
        let fileManage = FileManager.default
        fileManage.createFile(atPath: fileName, contents: data, attributes: nil)
        return fileName
    }
    
    private func getVideoThumbPath(url: String)->UIImage? {
        do {
          let avasset = AVAsset.init(url: NSURL.fileURL(withPath: url));
          let gen = AVAssetImageGenerator.init(asset: avasset);
          gen.appliesPreferredTrackTransform = true;
          let time = CMTime.init(seconds: 0.0, preferredTimescale: 600);
          let image = try gen.copyCGImage(at: time, actualTime: nil);
          let thumb = UIImage.init(cgImage: image);
          return thumb;
        } catch {
          return nil;
        }
      }
}
